package it.asterisk.pbx.internal.core;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Modifier;
import java.util.Hashtable;
import java.util.Set;

import it.asterisk.pbx.asterisk.wrap.actions.ManagerAction;
import it.asterisk.pbx.asterisk.wrap.events.ManagerEvent;
import it.asterisk.pbx.asterisk.wrap.events.ResponseEvent;
import it.asterisk.pbx.asterisk.wrap.response.CommandResponse;
import it.asterisk.pbx.asterisk.wrap.response.ManagerError;
import it.asterisk.pbx.asterisk.wrap.response.ManagerResponse;
import it.asterisk.util.ReflectionUtil;
import it.callite.log.AsteriskLog;

/**
 * This class maps asterisk-java events to our internal events that use iChannel
 * rather than raw channel names.
 *
 * @author bsutton
 */
@SuppressWarnings({"unchecked"})
public class CoherentEventFactory
{

    // Events
    static Hashtable<Class<it.asterisk.manager.event.ManagerEvent>, Class<ManagerEvent>> mapEvents = new Hashtable<>();

    // Response
    static Hashtable<Class< ? extends it.asterisk.manager.event.ResponseEvent>, Class< ? extends ResponseEvent>> mapResponses = new Hashtable<>();

    // static initialiser
    static
    {

        Set<Class<ManagerEvent>> knownClasses = ReflectionUtil.loadClasses("it.asterisk.pbx.asterisk.wrap.events",
                ManagerEvent.class);

        ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
        for (Class< ? > known : knownClasses)
        {
            Class< ? > clazz = null;
            try
            {
                clazz = classLoader.loadClass("it.asterisk.manager.event" + "." + known.getSimpleName());

                if (!Modifier.isAbstract(clazz.getModifiers()))
                {
                    if (known.getConstructor(clazz) != null)
                    {
                        if (ResponseEvent.class.isAssignableFrom(known))
                        {
                            CoherentEventFactory.mapResponses.put(
                                    (Class<it.asterisk.manager.event.ResponseEvent>) clazz,
                                    (Class<ResponseEvent>) known);
                            AsteriskLog.info("Response Event Added " + clazz + " --> " + known);

                        }
                        else
                        {
                            CoherentEventFactory.mapEvents.put((Class<it.asterisk.manager.event.ManagerEvent>) clazz,
                                    (Class<ManagerEvent>) known);
                            AsteriskLog.info("Manager Event Added " + clazz + " --> " + known);
                        }
                    }
                    else
                    {
                        AsteriskLog.warn("Skipping class " + clazz + " it doesn't have a public constructor with one arg of type "
                                + known);
                    }
                }
                else
                {
                    AsteriskLog.info("Skipping abstract class " + clazz);
                }
            }
            catch (ClassNotFoundException e)
            {
                AsteriskLog.error(e, e);
                throw new RuntimeException(e);
            }
            catch (NoSuchMethodException e)
            {
                AsteriskLog.error(clazz.getCanonicalName() + " doesn't have an appropriate event constructor");
            }
            catch (SecurityException e)
            {
                AsteriskLog.error(e, e);
            }
        }

        CoherentEventFactory.mapResponses.put(it.asterisk.manager.event.ResponseEvent.class, ResponseEvent.class);

    }

    public static Class< ? extends ManagerEvent> getShadowEvent(it.asterisk.manager.event.ManagerEvent event)
    {
        Class< ? extends ManagerEvent> result = CoherentEventFactory.mapEvents.get(event.getClass());
        if (result == null)
        {
            Class< ? extends ResponseEvent> response = CoherentEventFactory.mapResponses.get(event.getClass());
            result = response;
        }

        return result;

    }

    public static ManagerEvent build(final it.asterisk.manager.event.ManagerEvent event)
    {
        ManagerEvent iEvent = null;

        Class< ? extends ManagerEvent> target = null;

        if (event instanceof it.asterisk.manager.event.ResponseEvent)
            target = CoherentEventFactory.mapResponses.get(event.getClass());
        else
            target = CoherentEventFactory.mapEvents.get(event.getClass());

        if (target == null)
        {
            AsteriskLog.warn("The given event " + event.getClass().getName() + " is not supported "); //$NON-NLS-1$ //$NON-NLS-2$
        }
        else
        {

            try
            {
                iEvent = target.getDeclaredConstructor(event.getClass()).newInstance(event);
            }
            catch (NoSuchMethodException | SecurityException | InstantiationException | IllegalAccessException
                    | IllegalArgumentException | InvocationTargetException e)
            {
                AsteriskLog.error(e, e);

            }
        }
        return iEvent;
    }

    public static ResponseEvent build(it.asterisk.manager.event.ResponseEvent event)
    {
        ResponseEvent response = null;

        final Class< ? extends ResponseEvent> target = CoherentEventFactory.mapResponses.get(event.getClass());

        if (target == null)
        {
            AsteriskLog.warn("The given event " + event.getClass().getName() + " is not supported "); //$NON-NLS-1$ //$NON-NLS-2$
        }
        else
        {

            try
            {
                final Constructor< ? extends ResponseEvent> declaredConstructor = target
                        .getDeclaredConstructor(event.getClass());
                response = declaredConstructor.newInstance(event);
            }
            catch (NoSuchMethodException | SecurityException | InstantiationException | IllegalAccessException
                    | IllegalArgumentException | InvocationTargetException e)
            {
                AsteriskLog.error(e, e);

            }
        }
        return response;
    }

    public static ManagerResponse build(it.asterisk.manager.response.ManagerResponse response)
    {
        ManagerResponse result;
        if (response instanceof it.asterisk.manager.response.CommandResponse)
        {
            result = new CommandResponse(response);
        }
        else if (response instanceof it.asterisk.manager.response.ManagerError)
        {
            result = new ManagerError(response);
        }
        else
        {
            result = new ManagerResponse(response);
        }
        return result;

    }

    public static it.asterisk.manager.action.ManagerAction build(ManagerAction action)
    {
        it.asterisk.manager.action.ManagerAction result = null;

        // final Class<? extends it.asterisk.manager.action.ManagerAction>
        // target = CoherentEventFactory.mapActions.get(action.getClass());

         // AsteriskLog.debug("Action " + action); //$NON-NLS-1$

        // if (target == null)
        // {
        // AsteriskLog.warn("The given action " + action.getClass().getName() + " is
        // not supported "); //$NON-NLS-1$ //$NON-NLS-2$
        // }
        // else
        {
            result = action.getAJAction();
        }
        return result;

    }

}
