package it.asterisk.pbx.internal.managerAPI;

import java.util.HashSet;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import it.asterisk.pbx.AgiChannelActivityAction;
import it.asterisk.pbx.CallerID;
import it.asterisk.pbx.Channel;
import it.asterisk.pbx.EndPoint;
import it.asterisk.pbx.ListenerPriority;
import it.asterisk.pbx.NewChannelListener;
import it.asterisk.pbx.PBX;
import it.asterisk.pbx.PBXException;
import it.asterisk.pbx.PBXFactory;
import it.asterisk.pbx.asterisk.wrap.events.HangupEvent;
import it.asterisk.pbx.asterisk.wrap.events.ManagerEvent;
import it.asterisk.pbx.internal.core.AsteriskPBX;
import it.callite.log.AsteriskLog;

public class DialToAgi extends EventListenerBaseClass
{

    private final OriginateResult result[] = new OriginateResult[2];

    private volatile boolean hangupDetected = false;

    private OriginateToExtension originator;

    public DialToAgi(final String descriptiveName)
    {
        super(descriptiveName, PBXFactory.getActivePBX());
    }

    /**
     * Dials the targetEndPoint connecting the call to the localHandset. The
     * dial is done in two parts as we need to set the auto answer header
     * differently for each phone. For the localHandset we set auto answer on,
     * whilst for the remote targetEndPoint we MUST not auto answer the phone.
     * 
     * @param localHandset the users handset we are going to dial from (e.g.
     *            receptionist phone).
     * @param targetEndPoint - the remote handset we are going to connect the
     *            localHandset to.
     * @param dialContext - context we are going to dial from.
     * @param hideCallerId
     * @param channelVarsToSet
     * @param callerId - the callerID to display on the targetEndPoint.
     * @return
     * @throws PBXException
     */
    public OriginateResult[] dial(final NewChannelListener listener, final EndPoint localHandset,
            final AgiChannelActivityAction action, final CallerID callerID, Integer timeout, final boolean hideCallerId,
            Map<String, String> channelVarsToSet) throws PBXException, InterruptedException
    {
        final PBX pbx = PBXFactory.getActivePBX();

        try (final OriginateToExtension originate = new OriginateToExtension(listener))
        {
            this.startListener();
            originator = originate;

            // First bring the operator's handset up and connect it to the
            // 'njr-dial' extension where they can
            // wait whilst we complete the second leg
            final OriginateResult trcResult = originate.originate(localHandset, pbx.getExtensionAgi(), true,
                    ((AsteriskPBX) pbx).getManagementContext(), callerID, timeout, hideCallerId, channelVarsToSet);

            this.result[0] = trcResult;
            if (trcResult.isSuccess())
            {
                if (trcResult.getChannel() != null)
                {
                    trcResult.getChannel().setCurrentActivityAction(action);
                    if (trcResult.getChannel().waitForChannelToReachAgi(30, TimeUnit.SECONDS))
                    {
                        AsteriskLog.info("Call reached AGI", null);
                    }
                    else
                    {
                        AsteriskLog.error("Call never reached agi", null);
                    }
                }
                else
                {
                    AsteriskLog.error("Call never reached agi", null);
                }
            }
            else
            {
                AsteriskLog.warn("Originate failed: " + trcResult.getAbortReason(), null);
            }
            AsteriskLog.info("Hangup status is " + hangupDetected, null);

            return this.result;
        }
        finally
        {
            this.close();
        }
    }

    @Override
    public HashSet<Class< ? extends ManagerEvent>> requiredEvents()
    {
        HashSet<Class< ? extends ManagerEvent>> required = new HashSet<>();
        required.add(HangupEvent.class);

        return required;
    }

    @Override
    public void onManagerEvent(final ManagerEvent event)
    {
        if (event instanceof HangupEvent)
        {
            final HangupEvent hangup = (HangupEvent) event;
            final Channel hangupChannel = hangup.getChannel();

          /*
                AsteriskLog.debug("hangup :" + hangupChannel, null); //$NON-NLS-1$
                AsteriskLog.debug("channel 0:" + this.result[0], null); //$NON-NLS-1$
                AsteriskLog.debug("channel 1:" + this.result[1], null); //$NON-NLS-1$
            */
            if ((this.result[0] != null) && this.result[0].isSuccess() && hangupChannel.isSame(this.result[0].getChannel()))
            {
                hangupDetected = true;
            }
        }
    }

    @Override
    public ListenerPriority getPriority()
    {
        return ListenerPriority.NORMAL;
    }

    public void abort()
    {
        if (originator != null)
        {
            originator.abort("user abort");
        }
        else
        {
            AsteriskLog.error("Call to abort, but it doesn't look like the Dial had started yet", null);
        }
    }

}
