package it.callite.utility.xlsx;

import static it.callite.utility.xlsx.XmlUtils.document;
import static it.callite.utility.xlsx.XmlUtils.searchForNodeList;
import static it.callite.utility.xlsx.impl.TempFileUtil.writeInputStreamToFile;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.security.GeneralSecurityException;
import java.util.Iterator;
import java.util.Objects;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;

import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.openxml4j.exceptions.OpenXML4JException;
import org.apache.poi.openxml4j.opc.OPCPackage;
import org.apache.poi.poifs.crypt.Decryptor;
import org.apache.poi.poifs.crypt.EncryptionInfo;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.eventusermodel.XSSFReader;
import org.apache.poi.xssf.model.SharedStrings;
import org.apache.poi.xssf.model.StylesTable;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import it.callite.utility.xlsx.exceptions.MissingSheetException;
import it.callite.utility.xlsx.exceptions.OpenException;
import it.callite.utility.xlsx.exceptions.ReadException;
import it.callite.utility.xlsx.impl.StreamingSheetReader;
import it.callite.utility.xlsx.impl.StreamingWorkbook;
import it.callite.utility.xlsx.impl.StreamingWorkbookReader;
import it.callite.utility.xlsx.sst.BufferedStringsTable;


public class StreamingReader implements Iterable<Row>, AutoCloseable {

	/*
	<xlsx.version>2.2.0</xlsx.version>
	<dependency>
		<groupId>com.monitorjbl</groupId>
		<artifactId>xlsx-streamer</artifactId>
		<version>${xlsx.version}</version>
	</dependency>
	 */
	
	private File tmp;
	private final StreamingWorkbookReader workbook;

	public StreamingReader(StreamingWorkbookReader workbook) {
		this.workbook = workbook;
	}


	@Override
	public Iterator<Row> iterator() {
		return workbook.first().iterator();
	}


	@Override
	public void close() throws IOException {
		try {
			workbook.close();
		} finally {
			if(tmp != null) {
				tmp.delete();
			}
		}
	}

	public static Builder builder() {
		return new Builder();
	}

	public static class Builder {
		private int rowCacheSize = 10;
		private int bufferSize = 1024;
		private int sheetIndex = 0;
		private int sstCacheSize = -1;
		private String sheetName;
		private String password;

		public int getRowCacheSize() {
			return rowCacheSize;
		}

		public int getBufferSize() {
			return bufferSize;
		}

		public int getSheetIndex() {
			return sheetIndex;
		}


		public String getSheetName() {
			return sheetName;
		}

		public String getPassword() {
			return password;
		}


		public int getSstCacheSize() {
			return sstCacheSize;
		}


		public Builder rowCacheSize(int rowCacheSize) {
			this.rowCacheSize = rowCacheSize;
			return this;
		}


		public Builder bufferSize(int bufferSize) {
			this.bufferSize = bufferSize;
			return this;
		}


		public Builder sheetIndex(int sheetIndex) {
			this.sheetIndex = sheetIndex;
			return this;
		}


		public Builder sheetName(String sheetName) {
			this.sheetName = sheetName;
			return this;
		}


		public Builder password(String password) {
			this.password = password;
			return this;
		}


		public Builder sstCacheSize(int sstCacheSize) {
			this.sstCacheSize = sstCacheSize;
			return this;
		}


		public Workbook open(InputStream is) {
			StreamingWorkbookReader workbook = new StreamingWorkbookReader(this);
			workbook.init(is);
			return new StreamingWorkbook(workbook);
		}


		public Workbook open(File file) {
			StreamingWorkbookReader workbook = new StreamingWorkbookReader(this);
			workbook.init(file);
			return new StreamingWorkbook(workbook);
		}


		public StreamingReader read(InputStream is) {
			File f = null;
			try {
				f = writeInputStreamToFile(is, bufferSize);
				StreamingReader r = read(f);
				r.tmp = f;
				return r;
			} catch(IOException e) {
				throw new ReadException("Unable to read input stream", e);
			} catch(RuntimeException e) {
				if(f != null) {
					f.delete();
				}
				throw e;
			}
		}

		public StreamingReader read(File f) {
			try {
				OPCPackage pkg;
				if(password != null) {
					POIFSFileSystem poifs = new POIFSFileSystem(f);
					EncryptionInfo info = new EncryptionInfo(poifs);
					Decryptor d = Decryptor.getInstance(info);
					d.verifyPassword(password);
					pkg = OPCPackage.open(d.getDataStream(poifs));
				} else {
					pkg = OPCPackage.open(f);
				}

				boolean use1904Dates = false;
				XSSFReader reader = new XSSFReader(pkg);

				SharedStrings sst;
				File sstCache = null;
				if(sstCacheSize > 0) {
					sstCache = Files.createTempFile("", "").toFile();
					sst = BufferedStringsTable.getSharedStringsTable(sstCache, sstCacheSize, pkg);
				} else {
					sst = reader.getSharedStringsTable();
				}

				StylesTable styles = reader.getStylesTable();
				NodeList workbookPr = searchForNodeList(document(reader.getWorkbookData()), "/ss:workbook/ss:workbookPr");
				if (workbookPr.getLength() == 1) {
					final Node date1904 = workbookPr.item(0).getAttributes().getNamedItem("date1904");
					if (date1904 != null) {
						use1904Dates = ("1".equals(date1904.getTextContent()));
					}
				}
				InputStream sheet = findSheet(reader);
				if(sheet == null) {
					throw new MissingSheetException("Unable to find sheet at index [" + sheetIndex + "]");
				}

				XMLEventReader parser = XMLInputFactory.newInstance().createXMLEventReader(sheet);

				return new StreamingReader(new StreamingWorkbookReader(sst, sstCache, pkg, new StreamingSheetReader(sst, styles, parser, use1904Dates, rowCacheSize), this));
			} catch(IOException e) {
				throw new OpenException("Failed to open file", e);
			} catch(OpenXML4JException | XMLStreamException e) {
				throw new ReadException("Unable to read workbook", e);
			} catch(GeneralSecurityException e) {
				throw new ReadException("Unable to read workbook - Decryption failed", e);
			}
		}

		private InputStream findSheet(XSSFReader reader) throws IOException, InvalidFormatException {
			int index = sheetIndex;
			if(sheetName != null) {
				index = -1;
				NodeList nl = searchForNodeList(document(reader.getWorkbookData()), "/ss:workbook/ss:sheets/ss:sheet");
				for(int i = 0; i < nl.getLength(); i++) {
					if(Objects.equals(nl.item(i).getAttributes().getNamedItem("name").getTextContent(), sheetName)) {
						index = i;
					}
				}
				if(index < 0) {
					return null;
				}
			}
			Iterator<InputStream> iter = reader.getSheetsData();
			InputStream sheet = null;

			int i = 0;
			while(iter.hasNext()) {
				InputStream is = iter.next();
				if(i++ == index) {
					sheet = is;
					break;
				}
			}
			return sheet;
		}
	}

}
