package it.callite.utility.xlsx.sst;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.XMLEvent;

import org.apache.poi.openxml4j.opc.OPCPackage;
import org.apache.poi.openxml4j.opc.PackagePart;
import org.apache.poi.ss.usermodel.RichTextString;
import org.apache.poi.xssf.model.SharedStringsTable;
import org.apache.poi.xssf.usermodel.XSSFRelation;
import org.apache.poi.xssf.usermodel.XSSFRichTextString;

public class BufferedStringsTable extends SharedStringsTable implements AutoCloseable {
	private final FileBackedList list;

	public static BufferedStringsTable getSharedStringsTable(File tmp, int cacheSize, OPCPackage pkg) throws IOException {
		List<PackagePart> parts = pkg.getPartsByContentType(XSSFRelation.SHARED_STRINGS.getContentType());
		return parts.size() == 0 ? null : new BufferedStringsTable(parts.get(0), tmp, cacheSize);
	}

	private BufferedStringsTable(PackagePart part, File file, int cacheSize) throws IOException {
		this.list = new FileBackedList(file, cacheSize);
		readFrom(part.getInputStream());
	}

	@Override
	public void readFrom(InputStream is) throws IOException {
		try {
			XMLEventReader xmlEventReader = XMLInputFactory.newInstance().createXMLEventReader(is);

			while(xmlEventReader.hasNext()) {
				XMLEvent xmlEvent = xmlEventReader.nextEvent();

				if(xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equals("si")) {
					list.add(parseCT_Rst(xmlEventReader));
				}
			}
		} catch(XMLStreamException e) {
			throw new IOException(e);
		}
	}


	private String parseCT_Rst(XMLEventReader xmlEventReader) throws XMLStreamException {
		StringBuilder buf = new StringBuilder();
		XMLEvent xmlEvent;
		while((xmlEvent = xmlEventReader.nextTag()).isStartElement()) {
			switch(xmlEvent.asStartElement().getName().getLocalPart()) {
			case "t": 
				buf.append(xmlEventReader.getElementText());
				break;
			case "r":  
				parseCT_RElt(xmlEventReader, buf);
				break;
			case "rPh":  
			case "phoneticPr":
				skipElement(xmlEventReader);
				break;
			default:
				throw new IllegalArgumentException(xmlEvent.asStartElement().getName().getLocalPart());
			}
		}
		return buf.length() > 0 ? buf.toString() : null;
	}


	private void parseCT_RElt(XMLEventReader xmlEventReader, StringBuilder buf) throws XMLStreamException {
		XMLEvent xmlEvent;
		while((xmlEvent = xmlEventReader.nextTag()).isStartElement()) {
			switch(xmlEvent.asStartElement().getName().getLocalPart()) {
			case "t": // Text
				buf.append(xmlEventReader.getElementText());
				break;
			case "rPr": // Run Properties
				skipElement(xmlEventReader);
				break;
			default:
				throw new IllegalArgumentException(xmlEvent.asStartElement().getName().getLocalPart());
			}
		}
	}

	private void skipElement(XMLEventReader xmlEventReader) throws XMLStreamException {
		while(xmlEventReader.nextTag().isStartElement()) {
			skipElement(xmlEventReader);
		}
	}

	@Override
	public RichTextString getItemAt(int idx) {
		return new XSSFRichTextString(list.getAt(idx));
	}

	@Override
	public void close() throws IOException {
		super.close();
		list.close();
	}
}
